package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.util.UUIDUtils;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.*;
import com.usky.iot.mapper.BaseFacilityTypeMapper;
import com.usky.iot.mapper.BaseGgpFacilityMapper;
import com.usky.iot.service.*;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.service.vo.BaseGgpFacilityExportVo;
import com.usky.iot.service.vo.BaseGgpFacilityRequest;
import com.usky.iot.service.vo.BaseGgpFacilityTypeNumVO;
import com.usky.iot.service.vo.BaseGgpFacilityVO;
import com.usky.system.model.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 地理信息 服务实现类
 * </p>
 *
 * @author han
 * @since 2023-06-02
 */
@Service
public class BaseGgpFacilityServiceImpl extends AbstractCrudService<BaseGgpFacilityMapper, BaseGgpFacility> implements BaseGgpFacilityService {
    @Autowired
    private BaseFacilityDeviceService baseFacilityDeviceService;
    @Autowired
    private BaseFacilityBuildService baseFacilityBuildService;
    @Autowired
    private DmpDeviceInfoService dmpDeviceInfoService;
    @Autowired
    private BaseBuildService baseBuildService;
    @Autowired
    private BaseFacilityTypeMapper baseFacilityTypeMapper;

    @Override
    public boolean add(BaseGgpFacility baseGgpFacility) {
        List<BaseFacilityType> typeList = baseFacilityTypeMapper.typeList(baseGgpFacility.getFacilityType());
        baseGgpFacility.setFacilityNum(typeList.get(0).getTypeAbbrevia()+System.currentTimeMillis());
        baseGgpFacility.setCreator(SecurityUtils.getUsername());
        baseGgpFacility.setCreateTime(LocalDateTime.now());
        return this.save(baseGgpFacility);
    }

    @Override
    public void update(BaseGgpFacility baseGgpFacility) {
        baseGgpFacility.setUpdateTime(LocalDateTime.now());
        baseGgpFacility.setUpdatePerson(SecurityUtils.getUsername());
        this.updateById(baseGgpFacility);
    }

    @Override
    public CommonPage<BaseGgpFacility> page(BaseGgpFacilityRequest piRequest) {
        IPage<BaseGgpFacility> page = new Page<>(piRequest.getCurrent(), piRequest.getSize());
        LambdaQueryWrapper<BaseGgpFacility> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery
                .like(StringUtils.isNotBlank(piRequest.getFacilityName()),BaseGgpFacility::getFacilityName,
                        piRequest.getFacilityName())
                .eq(StringUtils.isNotBlank(piRequest.getFacilityNum()),BaseGgpFacility::getFacilityNum,piRequest.getFacilityNum())
                .eq(null != piRequest.getFacilityType(),BaseGgpFacility::getFacilityType,piRequest.getFacilityType())
                .eq(null != piRequest.getId(),BaseGgpFacility::getId,piRequest.getId())
//                .eq(BaseGgpFacility::getTenantId, SecurityUtils.getTenantId())
                .orderByDesc(BaseGgpFacility::getId);
        page = this.page(page,lambdaQuery);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public boolean remove(Integer id) {
        return this.removeById(id);
    }

    @Override
    public List<BaseGgpFacilityExportVo> recordLsitExport(BaseGgpFacilityRequest exRequest) {
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LambdaQueryWrapper<BaseGgpFacility> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
//                .eq(BaseGgpFacility::getTenantId, SecurityUtils.getTenantId())
                .like(StringUtils.isNotBlank(exRequest.getFacilityName()), BaseGgpFacility::getFacilityName, exRequest.getFacilityName())
                .eq(StringUtils.isNotBlank(exRequest.getFacilityNum()), BaseGgpFacility::getFacilityNum, exRequest.getFacilityNum())
                .eq(exRequest.getFacilityType() != 0,BaseGgpFacility::getFacilityType,exRequest.getFacilityType())
                .orderByDesc(BaseGgpFacility::getId);
        List<BaseGgpFacility> baseGgpFacilityList = this.list(queryWrapper);
        List<BaseGgpFacilityExportVo> list = new ArrayList<>();
        List<BaseGgpFacilityTypeNumVO> typeList = baseFacilityTypeMapper.typeNumList(1);
        if (CollectionUtils.isNotEmpty(baseGgpFacilityList)) {
            for (int i = 0; i < baseGgpFacilityList.size(); i++) {
                BaseGgpFacilityExportVo baseGgpFacilityExportVo = new BaseGgpFacilityExportVo();
                baseGgpFacilityExportVo.setXh(i + 1);
                baseGgpFacilityExportVo.setId(baseGgpFacilityList.get(i).getId());
                baseGgpFacilityExportVo.setFacilityNum(baseGgpFacilityList.get(i).getFacilityNum());
                baseGgpFacilityExportVo.setFacilityType("未知");
                if (CollectionUtils.isNotEmpty(typeList)) {
                    for (int j = 0; j < typeList.size(); j++) {
                        if (baseGgpFacilityList.get(i).getFacilityType().equals(typeList.get(j).getId())) {
                            baseGgpFacilityExportVo.setFacilityType(typeList.get(j).getTypeName());
                        }
                    }
                }

                baseGgpFacilityExportVo.setFacilityName(baseGgpFacilityList.get(i).getFacilityName());
                baseGgpFacilityExportVo.setAddress(baseGgpFacilityList.get(i).getAddress());
                baseGgpFacilityExportVo.setImagesUrl(baseGgpFacilityList.get(i).getImagesUrl());
                baseGgpFacilityExportVo.setContact(baseGgpFacilityList.get(i).getContact());
                baseGgpFacilityExportVo.setContactPhone(baseGgpFacilityList.get(i).getContactPhone());
                baseGgpFacilityExportVo.setStatus(baseGgpFacilityList.get(i).getStatus());
                baseGgpFacilityExportVo.setCreateTime(baseGgpFacilityList.get(i).getCreateTime().format(df));
                list.add(baseGgpFacilityExportVo);
            }
        }
        return list;
    }

    @Override
    public List<BaseGgpFacility> facilityInfo(Integer id) {
        LambdaQueryWrapper<BaseGgpFacility> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(null != id,BaseGgpFacility::getId, id);
        List<BaseGgpFacility> baseGgpFacilityList = this.list(queryWrapper);
        return baseGgpFacilityList;
    }

    @Override
    public CommonPage<DmpDeviceInfo> deviceInfo(String deviceId,Integer id,Integer current,Integer size) {
        IPage<DmpDeviceInfo> page = new Page<>(current, size);
        LambdaQueryWrapper<BaseFacilityDevice> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(BaseFacilityDevice::getFacilityId,id);
        List<BaseFacilityDevice> baseFacilityDeviceList = baseFacilityDeviceService.list(lambdaQuery);
        if (CollectionUtils.isNotEmpty(baseFacilityDeviceList)) {
            List<String> deviceIdList = new ArrayList<>();
            for (int i = 0; i < baseFacilityDeviceList.size(); i++) {
                deviceIdList.add(baseFacilityDeviceList.get(i).getDeviceId());
            }
            if (CollectionUtils.isNotEmpty(deviceIdList)) {
                if (StringUtils.isNotBlank(deviceId)){
                    LambdaQueryWrapper<DmpDeviceInfo> queryWrapper1 = Wrappers.lambdaQuery();
                    queryWrapper1.select(DmpDeviceInfo::getId, DmpDeviceInfo::getDeviceId,
                            DmpDeviceInfo::getDeviceName, DmpDeviceInfo::getInstallAddress)
                            .in(DmpDeviceInfo::getDeviceId, deviceIdList)
                            .eq(DmpDeviceInfo::getDeviceId, deviceId)
                            .eq(DmpDeviceInfo::getDeleteFlag,0);
                    page = dmpDeviceInfoService.page(page,queryWrapper1);
                }else {
                    LambdaQueryWrapper<DmpDeviceInfo> queryWrapper1 = Wrappers.lambdaQuery();
                    queryWrapper1.select(DmpDeviceInfo::getId, DmpDeviceInfo::getDeviceId,
                            DmpDeviceInfo::getDeviceName, DmpDeviceInfo::getInstallAddress)
                            .in(DmpDeviceInfo::getDeviceId, deviceIdList)
                            .eq(DmpDeviceInfo::getDeleteFlag,0);
                    page = dmpDeviceInfoService.page(page,queryWrapper1);
                }
            }
        }
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public CommonPage<Object> deviceBindInfo(String deviceId,Integer id,Integer current,Integer size) {
        List<Object> list = new ArrayList<>();
        IPage<DmpDeviceInfo> page = new Page<>(current, size);
        LambdaQueryWrapper<BaseFacilityDevice> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(BaseFacilityDevice::getFacilityId,id);
        List<BaseFacilityDevice> baseFacilityDeviceList = baseFacilityDeviceService.list(lambdaQuery);
        LambdaQueryWrapper<DmpDeviceInfo> queryWrapper1 = Wrappers.lambdaQuery();
        queryWrapper1.select(DmpDeviceInfo::getId, DmpDeviceInfo::getDeviceId,
                DmpDeviceInfo::getDeviceName, DmpDeviceInfo::getInstallAddress)
                .eq(StringUtils.isNotBlank(deviceId),DmpDeviceInfo::getDeviceId, deviceId)
                .eq(DmpDeviceInfo::getDeleteFlag,0);
        page = dmpDeviceInfoService.page(page,queryWrapper1);
        if (CollectionUtils.isNotEmpty(page.getRecords())) {
            for (int i = 0; i < page.getRecords().size(); i++) {
                Map<String, Object> map = new HashMap<>();
                map.put("id", page.getRecords().get(i).getId());
                map.put("deviceId", page.getRecords().get(i).getDeviceId());
                map.put("deviceName", page.getRecords().get(i).getDeviceName());
                map.put("installAddress", page.getRecords().get(i).getInstallAddress());
                map.put("bindStatus", 0);
                map.put("bindId", 0);
                if (CollectionUtils.isNotEmpty(baseFacilityDeviceList)) {
                    for (int j = 0; j < baseFacilityDeviceList.size(); j++) {
                        if (page.getRecords().get(i).getDeviceId().equals(baseFacilityDeviceList.get(j).getDeviceId())) {
                            map.put("bindStatus", 1);
                            map.put("bindId", baseFacilityDeviceList.get(j).getId());
                        }
                    }
                }
                list.add(map);
            }
        }
        return new CommonPage<>(list,page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public CommonPage<BaseBuild> buildInfo(String buildNum, Integer id, Integer current, Integer size) {
        IPage<BaseBuild> page = new Page<>(current, size);
        LambdaQueryWrapper<BaseFacilityBuild> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(BaseFacilityBuild::getFacilityId,id);
        List<BaseFacilityBuild> baseFacilityBuildList = baseFacilityBuildService.list(lambdaQuery);
        if (CollectionUtils.isNotEmpty(baseFacilityBuildList)) {
            List<Integer> buildIdList = new ArrayList<>();
            for (int i = 0; i < baseFacilityBuildList.size(); i++) {
                buildIdList.add(baseFacilityBuildList.get(i).getBuildId());
            }
            if (CollectionUtils.isNotEmpty(buildIdList)) {
                if (StringUtils.isNotBlank(buildNum)){
                    LambdaQueryWrapper<BaseBuild> queryWrapper1 = Wrappers.lambdaQuery();
                    queryWrapper1.select(BaseBuild::getId, BaseBuild::getBuildNum,
                            BaseBuild::getBuildName, BaseBuild::getAddress)
                            .in(BaseBuild::getId, buildIdList)
                            .eq(BaseBuild::getBuildNum, buildNum)
                            .eq(BaseBuild::getDeleteFlag,0);
                    page = baseBuildService.page(page,queryWrapper1);
                }else {
                    LambdaQueryWrapper<BaseBuild> queryWrapper1 = Wrappers.lambdaQuery();
                    queryWrapper1.select(BaseBuild::getId, BaseBuild::getBuildNum,
                            BaseBuild::getBuildName, BaseBuild::getAddress)
                            .in(BaseBuild::getId, buildIdList)
                            .eq(BaseBuild::getDeleteFlag,0);
                    page = baseBuildService.page(page,queryWrapper1);
                }
            }
        }
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public List<BaseGgpFacility> baseGgpFacilityList(BaseGgpFacilityVO baseGgpFacilityVO) {
        LambdaQueryWrapper<BaseGgpFacility> query = Wrappers.lambdaQuery();
        query.eq(StringUtils.isNotBlank(baseGgpFacilityVO.getStreetTown()),BaseGgpFacility::getStreetTown, baseGgpFacilityVO.getStreetTown());
        query.in(CollectionUtils.isNotEmpty(baseGgpFacilityVO.getFacilityType()),BaseGgpFacility::getFacilityType,
                baseGgpFacilityVO.getFacilityType());

        List<BaseGgpFacility> list = this.list(query);
        return list;
    }
}
