package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.domain.DmpProductInfo;
import com.usky.iot.mapper.DmpProductInfoMapper;
import com.usky.iot.service.DmpDeviceInfoService;
import com.usky.iot.service.DmpDeviceStatusService;
import com.usky.iot.service.DmpProductInfoService;
import com.usky.iot.service.vo.DmpProductInfoRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 产品信息表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-10-08
 */
@Service
public class DmpProductInfoServiceImpl extends AbstractCrudService<DmpProductInfoMapper, DmpProductInfo> implements DmpProductInfoService {


    @Autowired
    private DmpDeviceInfoService dmpDeviceInfoService;

    @Autowired
    private DmpDeviceStatusService dmpDeviceStatusService;
    @Override
    public boolean add(DmpProductInfo dmpProductInfo) {
        if (checkNameUnique(dmpProductInfo)){
            throw new BusinessException("新增产品信息'" + dmpProductInfo.getProductName() + "'失败，产品信息已存在");
        }
        return this.save(dmpProductInfo);
    }

    @Override
    public void update(DmpProductInfo dmpProductInfo) {
        dmpProductInfo.setUpdatedTime(new Date());
        if (checkNameUnique(dmpProductInfo)){
            throw new BusinessException("修改产品信息'" + dmpProductInfo.getProductName() + "'失败，产品信息已存在");
        }
        this.updateById(dmpProductInfo);
    }

    @Override
    public boolean checkNameUnique(DmpProductInfo dmpProductInfo) {
        Integer id = null == dmpProductInfo.getId() ? -1 : dmpProductInfo.getId();
        LambdaQueryWrapper<DmpProductInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DmpProductInfo::getProductName,dmpProductInfo.getProductName())
                .eq(DmpProductInfo::getDeleteFlag,0);
        DmpProductInfo one = this.getOne(queryWrapper);
        return null != one && !Objects.equals(one.getId(), id);
    }


    @Override
    public CommonPage<DmpProductInfo> page(DmpProductInfoRequest piRequest) {
        IPage<DmpProductInfo> page = new Page<>(piRequest.getCurrent(), piRequest.getSize());
        LambdaQueryWrapper<DmpProductInfo> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery
                .like(StringUtils.isNotBlank(piRequest.getProductName()),DmpProductInfo::getProductName,piRequest.getProductName())
                .eq(StringUtils.isNotBlank(piRequest.getDeviceModel()),DmpProductInfo::getDeviceModel,piRequest.getDeviceModel())
                .eq(null != piRequest.getId(),DmpProductInfo::getId,piRequest.getId())
                .eq(DmpProductInfo::getDeleteFlag,0);
        List<DmpProductInfo> records = this.list(lambdaQuery);
        List<Integer> productIds = records.stream().map(DmpProductInfo::getId).collect(Collectors.toList());
        List<Map<String, Object>> maps = dmpDeviceInfoService.deviceCollect(productIds);
        List<Map<String, Object>> collectByProduct = dmpDeviceStatusService.getCollectByProduct(productIds);

//        maps.forEach(s -> {enhanceByDevice(records,s);});
//        collectByProduct.forEach(s -> {enhanceByDeviceStatus(records,s);});
        if(records.size()>0){
            for(int i=0;i<records.size();i++){
                if(maps.size()>0){
                    for(int j=0;j<maps.size();j++){
                        if(maps.get(j).get("productId")==(records.get(i).getId())){
                            String coun = maps.get(j).get("count").toString();
                            Integer it = Integer.parseInt(maps.get(j).get("count").toString());
                            records.get(i).setDeviceCount(Integer.parseInt(maps.get(j).get("count").toString()));
                            records.get(i).setServiceStatusCount(Integer.parseInt(maps.get(j).get("serviceCount").toString()));
                            continue;
                        }
                    }
                }
                if(collectByProduct.size()>0){
                    for(int j=0;j<collectByProduct.size();j++){
                        if(collectByProduct.get(j).get("productId")==(records.get(i).getId())){
                            records.get(i).setDeviceStatusCount(Integer.parseInt(collectByProduct.get(j).get("deviceCount").toString()));
                            continue;
                        }
                    }
                }
            }

        }
        int total = 0;
        if(records.size()>0){
            total = records.size();
        }
        int current = 0;
        Integer pageNum = piRequest.getCurrent();
        Integer pageSize = piRequest.getSize();
        if(pageNum != null && pageSize > 0){
            current = (pageNum - 1)*pageSize;
        }
        List<DmpProductInfo> list = records.stream().skip(current).limit(pageSize).collect(Collectors.toList());

        return new CommonPage<>(list,total,page.getCurrent(),page.getSize());
    }

    public void enhanceByDevice(List<DmpProductInfo> dmpProductInfo, Map<String,Object> map){
        dmpProductInfo.stream()
                .filter(s -> map.get("productId").equals(s.getId().toString()))
                .findAny()
                .ifPresent(s -> {
                    s.setDeviceCount(Integer.parseInt(map.get("count").toString()));
                    s.setServiceStatusCount(Integer.parseInt(map.get("serviceCount").toString()));
                });

    }

    public void enhanceByDeviceStatus(List<DmpProductInfo> dmpProductInfo, Map<String,Object> map){
        dmpProductInfo.stream()
                .filter(s -> map.get("productId").equals(s.getId().toString()))
                .findAny()
                .ifPresent(s -> {
                    s.setDeviceStatusCount(Integer.parseInt(map.get("deviceCount").toString()));
                }
                );
    }

    @Override
    public boolean remove(Integer id) {
        int deviceCount = dmpDeviceInfoService.deviceCountByProductId(id);
        if (deviceCount > 0){
            throw new BusinessException("产品下存在设备,请先删除设备，再删除产品");
        }
        DmpProductInfo dmpProductInfo = this.getById(id);
        Optional.ofNullable(dmpProductInfo).orElseThrow(() -> new BusinessException("产品不存在"));
        dmpProductInfo.setDeleteFlag(1);
        return this.updateById(dmpProductInfo);
    }

    @Override
    public List<Map<String, Object>> collect() {
        return baseMapper.selectCollect();
    }
}
